@@ -323,119 +323,6 @@ rb_utf_friendly_time (time_t date)
 	return str;
 }
 
-#ifndef HAVE_COLLATE_KEY_FILENAME
-
-#define COLLATION_SENTINEL "\1\1\1"
-
-/* this is taked from glib 2.7/2.8, in case it is not present */
-gchar*
-rb_utf8_collate_key_for_filename (const gchar *str, gssize len)
-{
-	GString *result;
-	GString *append;
-	const gchar *p;
-	const gchar *prev;
-	gchar *collate_key;
-	gint digits;
-	gint leading_zeros;
-
-	if (len < 0)
-		len = strlen (str);
-
-	result = g_string_sized_new (len * 2);
-	append = g_string_sized_new (0);
-
-	/* No need to use utf8 functions, since we're only looking for ascii chars */
-	for (prev = p = str; *p != '\0'; p++) {
-		switch (*p) {
-		case '.':
-			if (prev != p) {
-				collate_key = g_utf8_collate_key (prev, p - prev);
-				g_string_append (result, collate_key);
-				g_free (collate_key);
-			}
-
-			g_string_append (result, COLLATION_SENTINEL "\1");
-
-			/* skip the dot */
-			prev = p + 1;
-			break;
-
-		case '0':
-		case '1':
-		case '2':
-		case '3':
-		case '4':
-		case '5':
-		case '6':
-		case '7':
-		case '8':
-		case '9':
-			if (prev != p) {
-				collate_key = g_utf8_collate_key (prev, p - prev);
-				g_string_append (result, collate_key);
-				g_free (collate_key);
-			}
-
-			g_string_append (result, COLLATION_SENTINEL "\2");
-			prev = p;
-
-			/* write d-1 colons */
-			if (*p == '0') {
-				leading_zeros = 1;
-				digits = 0;
-			} else {
-				leading_zeros = 0;
-				digits = 1;
-			}
-
-			do {
-				p++;
-
-				if (*p == '0' && !digits)
-					++leading_zeros;
-				else if (g_ascii_isdigit(*p))
-					++digits;
-				else
-					break;
-			} while (*p != '\0');
-
-			while (digits > 1) {
-				g_string_append_c (result, ':');
-				--digits;
-			}
-
-			if (leading_zeros > 0) {
-				g_string_append_c (append, (char)leading_zeros);
-				prev += leading_zeros;
-			}
-
-			/* write the number itself */
-			g_string_append_len (result, prev, p - prev);
-
-			prev = p;
-			--p;	  /* go one step back to avoid disturbing outer loop */
-			break;
-
-		default:
-			/* other characters just accumulate */
-			break;
-		}
-	}
-
-	if (prev != p) {
-		collate_key = g_utf8_collate_key (prev, p - prev);
-		g_string_append (result, collate_key);
-		g_free (collate_key);
-	}
-
-	g_string_append (result, append->str);
-	g_string_free (append, TRUE);
-
-	return g_string_free (result, FALSE);
-}
-#endif
-
 /* Copied from eel-vfs-extensions.c from eel CVS HEAD on 2004-05-09
  * This function is (C) 1999, 2000 Eazel, Inc.
  */
