@@ -1,4 +1,5 @@
-/*
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
  *  arch-tag: Implementation of totally random functions that didn't fit elsewhere
  *
  *  Copyright (C) 2003 Colin Walters <walters@verbum.org>
@@ -216,6 +217,13 @@ rb_compare_gtimeval (GTimeVal *a, GTimeVal *b)
 		return -1;
 }
 
+int
+rb_safe_strcmp (const char *a,
+                const char *b)
+{
+	return (!a && !b) ? 0 : (a && !b) || (!a && b) ? 1 : strcmp (a, b);
+}
+
 /* Taken from totem/video-utils.c CVS HEAD 2004-04-22 */
 static void
 totem_pixbuf_mirror (GdkPixbuf *pixbuf)
@@ -406,6 +414,7 @@ purge_useless_threads (gpointer data)
 
 
 static GStaticRecMutex rb_gdk_mutex;
+static gboolean mutex_recurses;
 
 static void
 _threads_enter (void)
@@ -419,9 +428,19 @@ _threads_leave (void)
 	g_static_rec_mutex_unlock (&rb_gdk_mutex);
 }
 
+
+void
+rb_assert_locked (GMutex *m)
+{
+	if (!mutex_recurses)
+		g_assert (!g_mutex_trylock (m));
+}
+
 void
 rb_threads_init (void)
 {
+	GMutex *m;
+
 	private_is_primary_thread = g_private_new (NULL);
 	g_private_set (private_is_primary_thread, GUINT_TO_POINTER (1));
 
@@ -429,6 +448,17 @@ rb_threads_init (void)
 	gdk_threads_set_lock_functions (_threads_enter, _threads_leave);
 	gdk_threads_init ();
 
+	m = g_mutex_new ();
+
+	g_mutex_lock (m);
+	mutex_recurses = g_mutex_trylock (m);
+	if (mutex_recurses)
+		g_mutex_unlock (m);
+	g_mutex_unlock (m);
+	g_mutex_free (m);
+
+	rb_debug ("GMutex %s recursive", mutex_recurses ? "is" : "isn't");
+
 	/* purge useless thread-pool threads occasionally */
 	g_timeout_add (30 * 1000, purge_useless_threads, NULL);
 }
@@ -441,12 +471,14 @@ rb_string_split_words (const gchar *string)
 	GSList *words, *current;
 	gunichar *unicode, *cur_write, *cur_read;
 	gchar **ret;
+	gchar *normalized;
 	gint i, wordcount = 1;
 	gboolean new_word = TRUE;
 
 	g_return_val_if_fail (string != NULL, NULL);
 
-	cur_write = cur_read = unicode = g_utf8_to_ucs4_fast (string, -1, NULL);
+	normalized = g_utf8_normalize(string, -1, G_NORMALIZE_DEFAULT);
+	cur_write = cur_read = unicode = g_utf8_to_ucs4_fast (normalized, -1, NULL);
 
 	/* we may fail here, we expect valid utf-8 */
 	g_return_val_if_fail (unicode != NULL, NULL);
@@ -531,6 +563,7 @@ rb_string_split_words (const gchar *string)
 
 	g_slist_free (words);
 	g_free (unicode);
+	g_free (normalized);
 
 	return ret;
 }
@@ -539,6 +572,7 @@ gchar*
 rb_search_fold (const char *original)
 {
 	GString *string;
+	gchar *normalized;
 	gunichar *unicode, *cur;
 	
 	g_return_val_if_fail (original != NULL, NULL);
@@ -546,7 +580,9 @@ rb_search_fold (const char *original)
 	/* old behaviour is equivalent to: return g_utf8_casefold (original, -1); */
 	
 	string = g_string_new (NULL);
-	unicode = g_utf8_to_ucs4_fast (original, -1, NULL);
+	normalized = g_utf8_normalize(original, -1, G_NORMALIZE_DEFAULT);
+	unicode = g_utf8_to_ucs4_fast (normalized, -1, NULL);
+	
 
 	for (cur = unicode; *cur != 0; cur++) {
 		switch (g_unichar_type (*cur)) {
@@ -593,6 +629,7 @@ rb_search_fold (const char *original)
 	}
 	
 	g_free (unicode);
+	g_free (normalized);
 			
 	return g_string_free (string, FALSE);
 }
@@ -725,13 +762,39 @@ rb_string_list_copy (GList *list)
 	return copy;
 }
 
+gboolean
+rb_string_list_contains (GList *list, const char *s)
+{
+	GList *l;
+
+	for (l = list; l != NULL; l = g_list_next (l)) {
+		if (strcmp ((const char *)l->data, s) == 0)
+			return TRUE;
+	}
+
+	return FALSE;
+}
+
+void
+rb_list_destroy_free (GList *list, GDestroyNotify destroyer)
+{
+	g_list_foreach (list, (GFunc)destroyer, NULL);
+	g_list_free (list);
+
+}
+
 void
 rb_list_deep_free (GList *list)
 {
-	g_list_foreach (list, (GFunc)g_free, NULL);
-	g_list_free (list);
+	rb_list_destroy_free (list, (GDestroyNotify)g_free);
 }
 
+void
+rb_slist_deep_free (GSList *list)
+{
+	g_slist_foreach (list, (GFunc)g_free, NULL);
+	g_slist_free (list);
+}
 
 static void
 collate_keys_cb (gpointer key, gpointer value, GList **list)
@@ -869,6 +932,21 @@ void
 rb_value_free (GValue *val)
 {
 	g_value_unset (val);
-	g_free (val);
+	g_slice_free (GValue, val);
 }
 
+gboolean
+rb_str_in_strv (const char *needle, char **haystack)
+{
+	int i;
+
+	if (needle == NULL || haystack == NULL)
+		return FALSE;
+
+	for (i = 0; haystack[i] != NULL; i++) {
+		if (strcmp (needle, haystack[i]) == 0)
+			return TRUE;
+	}
+
+	return FALSE;
+}
