@@ -57,6 +57,10 @@ typedef enum
 	RB_METADATA_FIELD_ALBUM_PEAK,		   /* double */
 	RB_METADATA_FIELD_LANGUAGE_CODE,	   /* string */
 	RB_METADATA_FIELD_MUSICBRAINZ_TRACKID,     /* string */
+	RB_METADATA_FIELD_MUSICBRAINZ_ARTISTID,    /* string */
+	RB_METADATA_FIELD_MUSICBRAINZ_ALBUMID,     /* string */
+	RB_METADATA_FIELD_MUSICBRAINZ_ALBUMARTISTID,   /* string */
+	RB_METADATA_FIELD_MUSICBRAINZ_ARTISTSORTNAME,  /* string */
 
 	RB_METADATA_FIELD_LAST			   /* nothing */
 } RBMetaDataField;
@@ -71,6 +75,7 @@ typedef enum
 	RB_METADATA_ERROR_INTERNAL,
 	RB_METADATA_ERROR_NOT_AUDIO,
 	RB_METADATA_ERROR_NOT_AUDIO_IGNORE,
+	RB_METADATA_ERROR_EMPTY_FILE
 } RBMetaDataError;
 
 #define RB_METADATA_ERROR rb_metadata_error_quark ()
