@@ -1,7 +1,7 @@
 /*
  *  arch-tag: Implementation of ipod source object
  *
- *  Copyright (C) 2004 Christophe Fergeau  <teuf@gnome.org>
+ *  Copyright (C) 2004, 2007 Christophe Fergeau  <teuf@gnome.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -36,6 +36,7 @@
 
 #include "eel-gconf-extensions.h"
 #include "rb-ipod-source.h"
+#include "rb-ipod-db.h"
 #include "rb-debug.h"
 #include "rb-file-helpers.h"
 #include "rb-plugin.h"
@@ -57,7 +58,6 @@ static void rb_ipod_source_dispose (GObject *object);
 static gboolean impl_show_popup (RBSource *source);
 static void impl_move_to_trash (RBSource *asource);
 static void rb_ipod_load_songs (RBiPodSource *source);
-static gchar *rb_ipod_get_mount_path (GnomeVFSVolume *volume);
 static void impl_delete_thyself (RBSource *source);
 static GList* impl_get_ui_actions (RBSource *source);
 #ifdef HAVE_HAL
@@ -81,13 +81,11 @@ static gchar* ipod_get_filename_for_uri (const gchar *mount_point,
 static gchar* ipod_path_from_unix_path (const gchar *mount_point,
 					const gchar *unix_path);
 #endif
-static void itdb_schedule_save (RBiPodSource *source);
 static RhythmDB *get_db_for_source (RBiPodSource *source);
 
 typedef struct
 {
-	Itdb_iTunesDB *ipod_db;
-	gchar *ipod_mount_path;
+	RbIpodDb *ipod_db;
 	GHashTable *entry_map;
 
 	gboolean needs_shuffle_db;
@@ -135,27 +133,13 @@ rb_ipod_source_class_init (RBiPodSourceClass *klass)
 	g_type_class_add_private (klass, sizeof (RBiPodSourcePrivate));
 }
 
+
 static void
 rb_ipod_source_set_ipod_name (RBiPodSource *source, const char *name)
 {
-	Itdb_Playlist *mpl;
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 
-	mpl = itdb_playlist_mpl (priv->ipod_db);
-	if (mpl != NULL) {
-		if (mpl->name != NULL) {
-			rb_debug ("Renaming iPod from %s to %s", mpl->name, name);
-			if (strcmp (mpl->name, name) == 0) {
-				rb_debug ("iPod is already named %s", name);
-				return;
-			}
-		}
-		g_free (mpl->name);
-		mpl->name = g_strdup (name);
-		itdb_schedule_save (source);
-	} else {
-		g_warning ("iPod's master playlist is missing");
-	}
+	rb_ipod_db_set_ipod_name (priv->ipod_db, name);
 }
 
 static void
@@ -171,7 +155,7 @@ rb_ipod_source_name_changed_cb (RBiPodSource *source, GParamSpec *spec,
 
 static void
 rb_ipod_source_init (RBiPodSource *source)
-{
+{	
 	g_signal_connect (G_OBJECT (source), "notify::name",
 			  (GCallback)rb_ipod_source_name_changed_cb, NULL);
 }
@@ -182,12 +166,9 @@ rb_ipod_source_constructor (GType type, guint n_construct_properties,
 {
 	RBiPodSource *source;
 	RBEntryView *songs;
-	RBiPodSourcePrivate *priv;
 
 	source = RB_IPOD_SOURCE (G_OBJECT_CLASS (rb_ipod_source_parent_class)->
 			constructor (type, n_construct_properties, construct_properties));
-	priv = IPOD_SOURCE_GET_PRIVATE (source);
-
 	songs = rb_source_get_entry_view (RB_SOURCE (source));
 	rb_entry_view_append_column (songs, RB_ENTRY_VIEW_COL_RATING, FALSE);
 	rb_entry_view_append_column (songs, RB_ENTRY_VIEW_COL_LAST_PLAYED, FALSE);
@@ -202,14 +183,9 @@ rb_ipod_source_dispose (GObject *object)
 {
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (object);
 
- 	if (priv->ipod_db != NULL) {
- 		itdb_free (priv->ipod_db);
- 		priv->ipod_db = NULL;
-	}
-
-	if (priv->ipod_mount_path) {
-		g_free (priv->ipod_mount_path);
-		priv->ipod_mount_path = NULL;
+	if (priv->ipod_db) {
+		g_object_unref (G_OBJECT (priv->ipod_db));
+		priv->ipod_db = NULL;
 	}
 
 	if (priv->entry_map) {
@@ -312,11 +288,13 @@ playlist_track_removed (RhythmDBQueryModel *m,
 	RBiPodSource *ipod = g_object_get_data (G_OBJECT (playlist), "ipod-source");
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (ipod);
 	Itdb_Track *track;
+
+	g_return_if_fail (ipod != NULL);
+	g_return_if_fail (ipod_pl != NULL);
+
         track = g_hash_table_lookup (priv->entry_map, entry);
 	g_return_if_fail (track != NULL);
-
-	itdb_playlist_remove_track (ipod_pl, track);
-	itdb_schedule_save (ipod);
+	rb_ipod_db_remove_from_playlist (priv->ipod_db, ipod_pl, track);
 }
 
 static void
@@ -330,12 +308,14 @@ playlist_track_added (GtkTreeModel *model, GtkTreePath *path,
 	Itdb_Track *track;
 	RhythmDBEntry *entry;
 
+	g_return_if_fail (ipod != NULL);
+	g_return_if_fail (ipod_pl != NULL);
+
 	gtk_tree_model_get (model, iter, 0, &entry, -1);
         track = g_hash_table_lookup (priv->entry_map, entry);
 	g_return_if_fail (track != NULL);
 
-	itdb_playlist_add_track (ipod_pl, track, -1);
-	itdb_schedule_save (ipod);
+	rb_ipod_db_add_to_playlist (priv->ipod_db, ipod_pl, track);
 }
 
 static void
@@ -346,7 +326,7 @@ add_rb_playlist (RBiPodSource *source, Itdb_Playlist *playlist)
 	GList *it;
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 	RhythmDBEntryType entry_type;
-		RhythmDBQueryModel *model;
+	RhythmDBQueryModel *model;
 
 	g_object_get (source,
 			  "shell", &shell,
@@ -362,34 +342,35 @@ add_rb_playlist (RBiPodSource *source, Itdb_Playlist *playlist)
 	for (it = playlist->members; it != NULL; it = it->next) {
 		Itdb_Track *song;
 		char *filename;
+		const char *mount_path;
 
 		song = (Itdb_Track *)it->data;
- 		filename = ipod_path_to_uri (priv->ipod_mount_path,
- 					    song->ipod_path);
+		mount_path = rb_ipod_db_get_mount_path (priv->ipod_db);
+ 		filename = ipod_path_to_uri (mount_path, song->ipod_path);
 		rb_static_playlist_source_add_location (RB_STATIC_PLAYLIST_SOURCE (playlist_source),
 							filename, -1);
 		g_free (filename);
 	}
 
 	g_object_ref (G_OBJECT(playlist_source));
-		playlist->userdata = playlist_source;
-		playlist->userdata_destroy = g_object_unref;
-		playlist->userdata_duplicate = g_object_ref;
+	playlist->userdata = playlist_source;
+	playlist->userdata_destroy = g_object_unref;
+	playlist->userdata_duplicate = g_object_ref;
 
-		model = rb_playlist_source_get_query_model (RB_PLAYLIST_SOURCE (playlist_source));
-		g_signal_connect (model, "row-inserted",
-				          G_CALLBACK (playlist_track_added),
-				          playlist_source);
-		g_signal_connect (model, "entry-removed",
-				          G_CALLBACK (playlist_track_removed),
-				          playlist_source);
+	model = rb_playlist_source_get_query_model (RB_PLAYLIST_SOURCE (playlist_source));
+	g_signal_connect (model, "row-inserted",
+			          G_CALLBACK (playlist_track_added),
+			          playlist_source);
+	g_signal_connect (model, "entry-removed",
+			          G_CALLBACK (playlist_track_removed),
+			          playlist_source);
 
-		g_object_set_data (G_OBJECT (playlist_source),
-				           "ipod-source", source);
-		g_object_set_data (G_OBJECT (playlist_source),
-				           "itdb-playlist", playlist);
-		if (itdb_playlist_is_podcasts(playlist))
-				priv->podcast_pl = RB_STATIC_PLAYLIST_SOURCE (playlist_source);
+	g_object_set_data (G_OBJECT (playlist_source),
+			           "ipod-source", source);
+	g_object_set_data (G_OBJECT (playlist_source),
+			           "itdb-playlist", playlist);
+	if (itdb_playlist_is_podcasts(playlist))
+			priv->podcast_pl = RB_STATIC_PLAYLIST_SOURCE (playlist_source);
 	rb_shell_append_source (shell, playlist_source, RB_SOURCE (source));
 	g_object_unref (shell);
 }
@@ -400,7 +381,9 @@ load_ipod_playlists (RBiPodSource *source)
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 	GList *it;
 
-	for (it = priv->ipod_db->playlists; it != NULL; it = it->next) {
+	for (it = rb_ipod_db_get_playlists (priv->ipod_db);
+	     it != NULL;
+	     it = it->next) {
 		Itdb_Playlist *playlist;
 
 		playlist = (Itdb_Playlist *)it->data;
@@ -445,6 +428,7 @@ create_ipod_song_from_entry (RhythmDBEntry *entry, const char *mimetype)
 	track->time_played = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_LAST_PLAYED);
 	track->time_played = itdb_time_host_to_mac (track->time_played);
 	track->rating = rhythmdb_entry_get_double (entry, RHYTHMDB_PROP_RATING);
+	track->rating *= ITDB_RATING_STEP;
 	track->app_rating = track->rating;
 	track->playcount = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_PLAY_COUNT);
 
@@ -465,13 +449,13 @@ add_ipod_song_to_db (RBiPodSource *source, RhythmDB *db, Itdb_Track *song)
 	RhythmDBEntryType entry_type;
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 	char *pc_path;
+	const char *mount_path;
 
 	/* Set URI */
 	g_object_get (source, "entry-type", &entry_type,
 		      NULL);
-
-	pc_path = ipod_path_to_uri (priv->ipod_mount_path,
-				    song->ipod_path);
+	mount_path = rb_ipod_db_get_mount_path (priv->ipod_db);
+	pc_path = ipod_path_to_uri (mount_path, song->ipod_path);
 	entry = rhythmdb_entry_new (RHYTHMDB (db), entry_type,
 				    pc_path);
 	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
@@ -577,7 +561,7 @@ add_ipod_song_to_db (RBiPodSource *source, RhythmDB *db, Itdb_Track *song)
 	if (song->rating != 0) {
 		GValue value = {0, };
 		g_value_init (&value, G_TYPE_DOUBLE);
-		g_value_set_double (&value, song->rating/20.0);
+		g_value_set_double (&value, song->rating/ITDB_RATING_STEP);
 		rhythmdb_entry_set (RHYTHMDB (db), entry,
 					       RHYTHMDB_PROP_RATING,
 					       &value);
@@ -627,6 +611,105 @@ get_db_for_source (RBiPodSource *source)
 	return db;
 }
 
+
+static void
+rb_ipod_source_entry_changed_cb (RhythmDB *db,
+				 RhythmDBEntry *entry,
+				 GSList *changes,
+				 RBiPodSource *source)
+{
+	GSList *t;
+
+	/* Ignore entries which are not iPod entries */
+	RhythmDBEntryType entry_type;
+	RhythmDBEntryType ipod_entry_type;
+	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
+
+	entry_type = rhythmdb_entry_get_pointer (entry, RHYTHMDB_PROP_TYPE);
+	g_object_get (G_OBJECT (source),
+		      "entry-type", &ipod_entry_type,
+		      NULL);
+	g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, ipod_entry_type);
+	/* ipod_entry_type can no longer be dereferenced, but its value 
+	 * hasn't been changed, so this comparison is valid
+	 */
+	if (entry_type != ipod_entry_type) {
+		return; 
+	};
+
+
+	/* If an interesting property was changed, update it on the iPod */
+	/* If the iPod database is being saved in a separate thread, this 
+	 * might not be 100% thread-safe, but at worse we'll modify a field
+	 * at the time it's being saved which will get a wrong value, but
+	 * that's the worse that can happen and that's pretty theoritical, 
+	 * I don't think avoiding it is worth the effort.
+	 */
+	for (t = changes; t; t = t->next) {
+		RhythmDBEntryChange *change = t->data;
+		switch (change->prop) {
+		case RHYTHMDB_PROP_RATING: {
+			Itdb_Track *track;
+			double old_rating;
+			double new_rating;
+
+			old_rating = g_value_get_double (&change->old);
+			new_rating = g_value_get_double (&change->new);
+			if (old_rating != new_rating) {
+				track = g_hash_table_lookup (priv->entry_map, 
+							     entry);
+				track->rating = new_rating * ITDB_RATING_STEP;
+				track->app_rating = track->rating;
+				rb_debug ("rating changed, saving db");
+				rb_ipod_db_save_async (priv->ipod_db);
+			} else {
+				rb_debug ("rating didn't change");
+			}
+			break;
+		}
+		case RHYTHMDB_PROP_PLAY_COUNT: {
+			Itdb_Track *track;
+			gulong old_playcount;
+			gulong new_playcount;
+
+			old_playcount = g_value_get_ulong (&change->old);
+			new_playcount = g_value_get_ulong (&change->new);
+			if (old_playcount != new_playcount) {
+				track = g_hash_table_lookup (priv->entry_map, 
+							     entry);
+				track->playcount = new_playcount;
+				rb_debug ("playcount changed, saving db");
+				rb_ipod_db_save_async (priv->ipod_db);
+			} else {
+				rb_debug ("playcount didn't change");
+			}
+			break;
+		}
+		case RHYTHMDB_PROP_LAST_PLAYED: {
+			Itdb_Track *track;
+			gulong old_lastplay;
+			gulong new_lastplay;
+
+			old_lastplay = g_value_get_ulong (&change->old);
+			new_lastplay = g_value_get_ulong (&change->new);
+			if (old_lastplay != new_lastplay) {
+				track = g_hash_table_lookup (priv->entry_map, 
+							     entry);
+				track->time_played = itdb_time_host_to_mac (new_lastplay);
+				rb_debug ("last play time changed, saving db");
+				rb_ipod_db_save_async (priv->ipod_db);
+			} else {
+				rb_debug ("last play time didn't change");
+			}
+			break;			
+		}
+		default:
+			rb_debug ("Ignoring property %d\n", change->prop);
+			break;
+		}
+	}
+}
+
 static gboolean
 load_ipod_db_idle_cb (RBiPodSource *source)
 {
@@ -639,12 +722,18 @@ load_ipod_db_idle_cb (RBiPodSource *source)
 	db = get_db_for_source (source);
 
 	g_assert (db != NULL);
- 	for (it = priv->ipod_db->tracks; it != NULL; it = it->next) {
+ 	for (it = rb_ipod_db_get_tracks (priv->ipod_db);
+	     it != NULL;
+	     it = it->next) {
 		add_ipod_song_to_db (source, db, (Itdb_Track *)it->data);
 	}
 
 	load_ipod_playlists (source);
 
+	g_signal_connect_object(G_OBJECT(db), "entry-changed", 
+				G_CALLBACK (rb_ipod_source_entry_changed_cb),
+				source, 0);
+
 	g_object_unref (db);
 
 	GDK_THREADS_LEAVE ();
@@ -657,54 +746,24 @@ rb_ipod_load_songs (RBiPodSource *source)
 {
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 	GnomeVFSVolume *volume;
-	const Itdb_IpodInfo *info;
 
 	g_object_get (source, "volume", &volume, NULL);
-	priv->ipod_mount_path = rb_ipod_get_mount_path (volume);
-
- 	priv->ipod_db = itdb_parse (priv->ipod_mount_path, NULL);
-
-	info = itdb_device_get_ipod_info(priv->ipod_db->device);
-	if (info->ipod_generation == ITDB_IPOD_GENERATION_UNKNOWN ||
-	    info->ipod_model == ITDB_IPOD_MODEL_SHUFFLE) {
-		priv->needs_shuffle_db = TRUE;
-	} else {
-		priv->needs_shuffle_db = FALSE;
-	}
-
+ 	priv->ipod_db = rb_ipod_db_new (volume);
 	priv->entry_map = g_hash_table_new (g_direct_hash, g_direct_equal);
-	if ((priv->ipod_db != NULL) && (priv->entry_map != NULL)) {
-		Itdb_Playlist *mpl;
 
-		/* FIXME: we could set a different icon depending on the iPod
-		 * model
-		 */
-		mpl = itdb_playlist_mpl (priv->ipod_db);
-		if (mpl && mpl->name) {
+	if ((priv->ipod_db != NULL) && (priv->entry_map != NULL)) {
+		const char *name;
+		name = rb_ipod_db_get_ipod_name (priv->ipod_db);
+		if (name) {
 			g_object_set (RB_SOURCE (source),
-				      "name", mpl->name,
+				      "name", name,
 				      NULL);
 		}
 		priv->load_idle_id = g_idle_add ((GSourceFunc)load_ipod_db_idle_cb, source);
 	}
-
 	g_object_unref (volume);
 }
 
-static gchar *
-rb_ipod_get_mount_path (GnomeVFSVolume *volume)
-{
-	gchar *path;
-	gchar *uri;
-
-	uri = gnome_vfs_volume_get_activation_uri (volume);
-	path = g_filename_from_uri (uri, NULL, NULL);
-	g_assert (path != NULL);
-	g_free (uri);
-
-	return path;
-}
-
 static gchar *
 rb_ipod_get_itunesdb_path (GnomeVFSVolume *volume)
 {
@@ -861,17 +920,6 @@ impl_show_popup (RBSource *source)
 	return TRUE;
 }
 
-static void
-remove_track_from_db (Itdb_Track *track)
-{
-	GList *it;
-
-	for (it = track->itdb->playlists; it != NULL; it = it->next) {
-		itdb_playlist_remove_track ((Itdb_Playlist *)it->data, track);
-	}
-	itdb_track_remove (track);
-}
-
 static void
 impl_move_to_trash (RBSource *asource)
 {
@@ -899,40 +947,17 @@ impl_move_to_trash (RBSource *asource)
 			continue;
 		}
 
- 		remove_track_from_db (track);
+		rb_ipod_db_remove_track (priv->ipod_db, track);
 		g_hash_table_remove (priv->entry_map, entry);
 		rhythmdb_entry_move_to_trash (db, entry);
 		rhythmdb_commit (db);
 	}
 
-	if (sel != NULL) {
-		itdb_schedule_save (RB_IPOD_SOURCE (asource));
-	}
-
 	g_object_unref (db);
 
 	g_list_free (sel);
 }
 
-static void
-itdb_schedule_save (RBiPodSource *source)
-{
-	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
-
-	/* FIXME: should probably be delayed a bit to avoid doing
-	 * it after each file when we are copying several files
-	 * consecutively
-	 * FIXME: or this function could be called itdb_set_dirty, and we'd
-	 * have a timeout firing every 5 seconds and saving the db if it's
-	 * dirty
-	 */
-	itdb_write (priv->ipod_db, NULL);
-
-	if (priv->needs_shuffle_db) {
-		itdb_shuffle_write (priv->ipod_db, NULL);
-	}
-}
-
 #ifdef ENABLE_IPOD_WRITING
 static char *
 impl_build_dest_uri (RBRemovableMediaSource *source,
@@ -943,9 +968,12 @@ impl_build_dest_uri (RBRemovableMediaSource *source,
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 	const char *uri;
 	char *dest;
+	const char *mount_path;
 
 	uri = rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION);
-	dest = ipod_get_filename_for_uri (priv->ipod_mount_path,  uri, mimetype, extension);
+	mount_path = rb_ipod_db_get_mount_path (priv->ipod_db);
+	dest = ipod_get_filename_for_uri (mount_path,  uri, 
+					  mimetype, extension);
 	if (dest != NULL) {
 		char *dest_uri;
 
@@ -966,39 +994,17 @@ artwork_notify_cb (RhythmDB *db,
 {
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (isource);
 	Itdb_Track *song;
-#ifdef HAVE_ITDB_TRACK_SET_THUMBNAILS_FROM_PIXBUF
-#else /* HAVE_ITDB_TRACK_SET_THUMBNAILS_FROM_PIXBUF */
-	gchar *image_data;
-	gsize image_data_len;
-	GError *err = NULL;
-	gboolean success;
-#endif /* HAVE_ITDB_TRACK_SET_THUMBNAILS_FROM_PIXBUF */
-	
+	GdkPixbuf *pixbuf;
+
+	g_return_if_fail (G_VALUE_HOLDS (metadata, GDK_TYPE_PIXBUF));
+	pixbuf = GDK_PIXBUF (g_value_get_object (metadata));
+		
 	song = g_hash_table_lookup (priv->artwork_request_map, entry);
 	if (song == NULL)
 		return;
 
-	g_return_if_fail (G_VALUE_HOLDS (metadata, GDK_TYPE_PIXBUF));
-#ifdef HAVE_ITDB_TRACK_SET_THUMBNAILS_FROM_PIXBUF
-	itdb_track_set_thumbnails_from_pixbuf (song, g_value_get_object (metadata));
-#else /* HAVE_ITDB_TRACK_SET_THUMBNAILS_FROM_PIXBUF */
-	success = gdk_pixbuf_save_to_buffer (GDK_PIXBUF (g_value_get_object (metadata)),
-					     &image_data, &image_data_len,
-					     "jpeg", &err,
-					     "quality", "100",
-					     NULL);
-	if (!success) {
-		g_assert (image_data == NULL);
-		g_warning ("Failed to save pixbuf to buffer %s", err->message);
-		g_error_free (err);
-		return;
-	}
-
+	rb_ipod_db_set_thumbnail (priv->ipod_db, song, pixbuf);
 	g_hash_table_remove (priv->artwork_request_map, entry);
-	itdb_track_set_thumbnails_from_data (song, (guchar *) image_data, image_data_len);
-	g_free (image_data);
-#endif /* HAVE_ITDB_TRACK_SET_THUMBNAILS_FROM_PIXBUF */
-	itdb_schedule_save (isource);
 }
 
 static void
@@ -1032,17 +1038,19 @@ add_to_podcasts (RBiPodSource *source, Itdb_Track *song)
 {
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 	gchar *filename;
+	const gchar *mount_path;
 
 	if (priv->podcast_pl == NULL) {
 		/* No Podcast playlist on the iPod, create a new one */
 		Itdb_Playlist *ipod_playlist;
 		ipod_playlist = itdb_playlist_new (_("Podcasts"), FALSE);
 		itdb_playlist_set_podcasts (ipod_playlist);
-		itdb_playlist_add (priv->ipod_db, ipod_playlist, -1);
+		rb_ipod_db_add_playlist (priv->ipod_db, ipod_playlist);
 		add_rb_playlist (source, ipod_playlist);
 	}
 
-  	filename = ipod_path_to_uri (priv->ipod_mount_path, song->ipod_path);
+	mount_path = rb_ipod_db_get_mount_path (priv->ipod_db);
+  	filename = ipod_path_to_uri (mount_path, song->ipod_path);
  	rb_static_playlist_source_add_location (priv->podcast_pl, filename, -1);
 	g_free (filename);
 }
@@ -1063,13 +1071,14 @@ impl_track_added (RBRemovableMediaSource *source,
 	if (song != NULL) {
 		RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 		char *filename;
+		const char *mount_path;
 
 		filename = g_filename_from_uri (dest, NULL, NULL);
-		song->ipod_path = ipod_path_from_unix_path (priv->ipod_mount_path, filename);
+		mount_path = rb_ipod_db_get_mount_path (priv->ipod_db);
+		song->ipod_path = ipod_path_from_unix_path (mount_path,
+							    filename);
 		g_free (filename);
-		itdb_track_add (priv->ipod_db, song, -1);
-		itdb_playlist_add_track (itdb_playlist_mpl (priv->ipod_db),
-					 song, -1);
+
 		if (song->mediatype == MEDIATYPE_PODCAST) {
 			add_to_podcasts (isource, song);
 		}
@@ -1077,14 +1086,16 @@ impl_track_added (RBRemovableMediaSource *source,
 		/* reuse that #define since both functions were added to 
 		 * libgpod CVS HEAD around the same time
 		 */
-		if (itdb_device_supports_artwork (priv->ipod_db->device)) {
+		Itdb_Device *device;
+		device = rb_ipod_db_get_device (priv->ipod_db);		
+		if (device && itdb_device_supports_artwork (device)) {
 			request_artwork (isource, entry, db, song);
 		}
 #else 
 		request_artwork (isource, entry, db, song);
 #endif
 		add_ipod_song_to_db (isource, db, song);
-		itdb_schedule_save (isource);
+		rb_ipod_db_add_track (priv->ipod_db, song);
 	}
 
 	g_object_unref (db);
@@ -1328,16 +1339,32 @@ impl_delete_thyself (RBSource *source)
 	RBiPodSourcePrivate *priv = IPOD_SOURCE_GET_PRIVATE (source);
 	GList *p;
 
-	for (p = priv->ipod_db->playlists; p != NULL; p = p->next) {
+	for (p = rb_ipod_db_get_playlists (priv->ipod_db);
+	     p != NULL;
+	     p = p->next) {
 		Itdb_Playlist *playlist = (Itdb_Playlist *)p->data;
-		if (!itdb_playlist_is_mpl (playlist)) {
-			RBSource *rb_playlist = RB_SOURCE (playlist->userdata);
+		if (!itdb_playlist_is_mpl (playlist) && !playlist->is_spl) {
+			RBSource *rb_playlist;
+			RhythmDBQueryModel *model;
+
+			rb_playlist = RB_SOURCE (playlist->userdata);
+			model = rb_playlist_source_get_query_model (RB_PLAYLIST_SOURCE (rb_playlist));
+
+			/* remove these to ensure they aren't called during source deletion */
+			g_signal_handlers_disconnect_by_func (model,
+							      G_CALLBACK (playlist_track_added),
+							      rb_playlist);
+			g_signal_handlers_disconnect_by_func (model,
+							      G_CALLBACK (playlist_track_removed),
+							      rb_playlist);
 			rb_source_delete_thyself (rb_playlist);
 		}
 	}
 
-	itdb_free (priv->ipod_db);
-	priv->ipod_db = NULL;
+	if (priv->ipod_db != NULL) {
+		g_object_unref (G_OBJECT (priv->ipod_db));
+		priv->ipod_db = NULL;
+	}
 
 	RB_SOURCE_CLASS (rb_ipod_source_parent_class)->impl_delete_thyself (source);
 }
