@@ -177,21 +177,35 @@ class MagnatuneSource(rb.BrowserSource):
 		urls = set([])
 
 		for tr in tracks:
-			url = self.__home_dict[self.__db.entry_get(tr, rhythmdb.PROP_LOCATION)]
+			sku = self.__sku_dict[self.__db.entry_get(tr, rhythmdb.PROP_LOCATION)]
+			url = self.__home_dict[sku]
 			if url not in urls:
 				gnomevfs.url_show(url)
 				urls.add(url)
-	
+
 	def buy_cd(self):
 		tracks = self.get_entry_view().get_selected_entries()
 		urls = set([])
 
 		for tr in tracks:
-			url = self.__buy_dict[self.__db.entry_get(tr, rhythmdb.PROP_LOCATION)]
+			sku = self.__sku_dict[self.__db.entry_get(tr, rhythmdb.PROP_LOCATION)]
+			url = self.__buy_dict[sku]
 			if url not in urls:
 				gnomevfs.url_show(url)
 				urls.add(url)
 
+	def radio_toggled(self, gladexml):
+		gc = gladexml.get_widget("radio_gc").get_active()
+		gladexml.get_widget("remember_cc_details").set_sensitive(not gc)
+		gladexml.get_widget("name_entry").set_sensitive(not gc)
+		gladexml.get_widget("cc_entry").set_sensitive(not gc)
+		gladexml.get_widget("mm_entry").set_sensitive(not gc)
+		gladexml.get_widget("yy_entry").set_sensitive(not gc)
+		
+		gladexml.get_widget("gc_entry").set_sensitive(gc)
+		if not gc:
+			gladexml.get_widget("gc_entry").set_text("")
+	
 	def purchase_album(self):
 		try:
 			library_location = self.__client.get_list("/apps/rhythmbox/library_locations", gconf.VALUE_STRING)[0] # Just use the first library location
@@ -212,7 +226,10 @@ class MagnatuneSource(rb.BrowserSource):
 			album = self.__db.entry_get(track, rhythmdb.PROP_ALBUM)
 
 			gladexml = gtk.glade.XML(self.__plugin.find_file("magnatune-purchase.glade"))
-
+			cb_dict = {"rb_magnatune_on_radio_cc_toggled_cb":lambda w:self.radio_toggled(gladexml)}
+			gladexml.signal_autoconnect(cb_dict)
+			
+			gladexml.get_widget("gc_entry").set_sensitive(False)
 			gladexml.get_widget("pay_combobox").set_active(self.__client.get_int(self.__plugin.gconf_keys['pay']) - 5)
 			gladexml.get_widget("audio_combobox").set_active(self.__plugin.format_list.index(self.__client.get_string(self.__plugin.gconf_keys['format'])))
 			gladexml.get_widget("info_label").set_markup(_("Would you like to purchase the album <i>%(album)s</i> by '%(artist)s'?") % {"album":album, "artist":artist})
@@ -238,7 +255,6 @@ class MagnatuneSource(rb.BrowserSource):
 
 				gladexml.get_widget("remember_cc_details").set_active(False)
 
-
 			window = gladexml.get_widget("purchase_dialog")
 			if window.run() == gtk.RESPONSE_ACCEPT:
 				amount = gladexml.get_widget("pay_combobox").get_active() + 5
@@ -248,13 +264,15 @@ class MagnatuneSource(rb.BrowserSource):
 				ccmonth = str(gladexml.get_widget("mm_entry").get_active() + 1).zfill(2)
 				name = gladexml.get_widget("name_entry").get_text()
 				email = gladexml.get_widget("email_entry").get_text()
+				gc = gladexml.get_widget("radio_gc").get_active()
+				gc_text = gladexml.get_widget("gc_entry").get_text()
 
 				if gladexml.get_widget("remember_cc_details").props.active:
 					self.__plugin.store_cc_details(ccnumber, ccyear, ccmonth, name, email)
 				else:
 					self.__plugin.clear_cc_details()
 
-				self.__buy_album (sku, amount, format, ccnumber, ccyear, ccmonth, name, email)
+				self.__buy_album (sku, amount, format, ccnumber, ccyear, ccmonth, name, email, gc, gc_text)
 
 			window.destroy()
 
@@ -388,8 +406,8 @@ class MagnatuneSource(rb.BrowserSource):
 	def __show_loading_screen(self, show):
 		if self.__info_screen is None:
 			# load the glade stuff
-			gladexml = gtk.glade.XML(self.__plugin.find_file("magnatune-loading.glade"), root="magnatune_loading_vbox")
-			self.__info_screen = gladexml.get_widget("magnatune_loading_vbox")
+			gladexml = gtk.glade.XML(self.__plugin.find_file("magnatune-loading.glade"), root="magnatune_loading_scrolledwindow")
+			self.__info_screen = gladexml.get_widget("magnatune_loading_scrolledwindow")
 			self.pack_start(self.__info_screen)
 			self.get_entry_view().set_no_show_all (True)
 			self.__info_screen.set_no_show_all (True)
@@ -409,19 +427,23 @@ class MagnatuneSource(rb.BrowserSource):
 	#
 	# internal purchasing code
 	#
-	def __buy_album(self, sku, pay, format, ccnumber, ccyear, ccmonth, name, email): # http://magnatune.com/info/api#purchase
+	def __buy_album(self, sku, pay, format, ccnumber, ccyear, ccmonth, name, email, gc, gc_text): # http://magnatune.com/info/api#purchase
 		print "purchasing tracks:", sku, pay, format, name, email
-		url = "https://magnatune.com/buy/buy_dl_cc_xml?"
-		url = url + urllib.urlencode({
+		url_dict = {
 						'id':	magnatune_partner_id,
 						'sku':	sku,
 						'amount': pay,
-						'cc':	ccnumber,
-						'yy':	ccyear,
-						'mm':	ccmonth,
 						'name': name,
 						'email':email
-					})
+			}
+		if gc:
+			url_dict['gc'] = gc
+		else:
+			url_dict['cc'] = ccnumber
+			url_dict['yy'] = ccyear
+			url_dict['mm'] = ccmonth
+		url = "https://magnatune.com/buy/buy_dl_cc_xml?"
+		url = url + urllib.urlencode(url_dict)
 
 		buy_album_handler = BuyAlbumHandler(format) # so we can get the url and auth info
 		auth_parser = xml.sax.make_parser()
@@ -567,10 +589,14 @@ class MagnatuneSource(rb.BrowserSource):
 
 		if entry.get_entry_type() != self.__db.entry_type_get_by_name("MagnatuneEntryType"):
 			return
-		
-		url = str(self.__art_dict[self.__db.entry_get(entry, rhythmdb.PROP_LOCATION)])
-		
+
+		gobject.idle_add (self.emit_cover_art_uri, entry)
+
+	def emit_cover_art_uri (self, entry):
+		sku = self.__sku_dict[self.__db.entry_get(entry, rhythmdb.PROP_LOCATION)]
+		url = self.__art_dict[sku]
 		self.__db.emit_entry_extra_metadata_notify (entry, 'rb:coverArt-uri', url)
+		return False
 
 gobject.type_register(MagnatuneSource)
 
