@@ -68,7 +68,6 @@
 #include "rb-preferences.h"
 #include "eel-gconf-extensions.h"
 
-/*#include "visualization-icon.h"*/
 #define VISUALIZATION_ICON_NAME	"visualization"
 
 #include <dbus/dbus-glib.h>
@@ -396,6 +395,8 @@ bus_sync_message_cb (GstBus *bus, GstMessage *msg, RBVisualizerPlugin *plugin)
 		return;
 	}
 
+	rb_debug ("handling prepare-xwindow-id message");
+
 	/* update our idea of where the xoverlay is */
 	find_xoverlay (plugin);
 
@@ -431,6 +432,7 @@ fixate_vis_caps (RBVisualizerPlugin *pi, GstElement *vis_element, GstElement *ca
 {
 	GstPad *pad;
 	GstCaps *caps = NULL;
+	const GstCaps *template_caps;
 	const VisualizerQuality *q;
 
 	if (quality < 0)
@@ -445,13 +447,23 @@ fixate_vis_caps (RBVisualizerPlugin *pi, GstElement *vis_element, GstElement *ca
 		q = &fake_vis_quality;
 
 	pad = gst_element_get_pad (vis_element, "src");
-	caps = gst_pad_get_allowed_caps (pad);
+	template_caps = gst_pad_get_pad_template_caps (pad);
 	gst_object_unref (pad);
 
-	if (caps && !gst_caps_is_fixed (caps)) {
+	if (template_caps == NULL) {
+		rb_debug ("vis element has no template caps?");
+		return;
+	}
+
+	caps = gst_caps_copy (template_caps);
+
+	if (gst_caps_is_fixed (caps) == FALSE) {
 		guint i;
 		char *dbg;
 
+		rb_debug ("fixating caps towards %dx%d, %d/%d",
+			  q->width, q->height,
+			  q->fps_n, q->fps_d);
 		caps = gst_caps_make_writable (caps);
 		for (i = 0; i < gst_caps_get_size (caps); i++) {
 			GstStructure *s = gst_caps_get_structure (caps, i);
@@ -466,16 +478,13 @@ fixate_vis_caps (RBVisualizerPlugin *pi, GstElement *vis_element, GstElement *ca
 		g_free (dbg);
 
 		g_object_set (capsfilter, "caps", caps, NULL);
-	} else if (caps) {
+	} else {
 		char *dbg = gst_caps_to_string (caps);
 		rb_debug ("vis element caps already fixed: %s", dbg);
 		g_free (dbg);
-	} else {
-		rb_debug ("vis element has no caps");
 	}
 
-	if (GST_IS_CAPS (caps))
-		gst_caps_unref (caps);
+	gst_caps_unref (caps);
 }
 
 static GstElement *
@@ -546,6 +555,65 @@ update_playbin_visualizer (RBVisualizerPlugin *plugin,
 	g_object_set (plugin->playbin, "vis-plugin", plugin->visualizer, NULL);
 }
 
+static void
+tee_visualizer_inserted (RBPlayerGstTee *player, GstElement *tee, RBVisualizerPlugin *plugin)
+{
+	GstBus *bus;
+	GstElement *p = tee;
+
+	if (tee != plugin->visualizer)
+		return;
+
+	rb_debug ("tee visualizer inserted into pipeline");
+
+	/* find the pipeline, extract its bus */
+	while (GST_ELEMENT_PARENT (p) != NULL) {
+		p = GST_ELEMENT_PARENT (p);
+	}
+	bus = gst_element_get_bus (p);
+
+	/* XXX sort of nasty, but avoids a g_warning */
+	if (bus->sync_handler == NULL) {
+		gst_bus_set_sync_handler (bus, gst_bus_sync_signal_handler, plugin);
+	}
+
+	if (plugin->bus_sync_id == 0) {
+		plugin->bus_sync_id = g_signal_connect (bus,
+							"sync-message::element",
+							G_CALLBACK (bus_sync_message_cb),
+							plugin);
+		rb_debug ("connected signal handler %d to bus %p", plugin->bus_sync_id, bus);
+	}
+
+	gst_object_unref (bus);
+}
+
+static void
+tee_visualizer_pre_remove (RBPlayerGstTee *player, GstElement *tee, RBVisualizerPlugin *plugin)
+{
+	GstBus *bus;
+	GstElement *p = tee;
+
+	if (tee != plugin->visualizer)
+		return;
+
+	rb_debug ("tee visualizer about to be removed from pipeline");
+
+	/* find the pipeline, extract its bus */
+	while (GST_ELEMENT_PARENT (p) != NULL) {
+		p = GST_ELEMENT_PARENT (p);
+	}
+	bus = gst_element_get_bus (p);
+
+	if (plugin->bus_sync_id != 0) {
+		rb_debug ("disconnecting handler %d from bus %p", plugin->bus_sync_id, bus);
+		g_signal_handler_disconnect (bus, plugin->bus_sync_id);
+		plugin->bus_sync_id = 0;
+	}
+
+	gst_object_unref (bus);
+}
+
 static void
 update_tee_visualizer (RBVisualizerPlugin *plugin,
 		       const char *vis_override,
@@ -1466,6 +1534,15 @@ impl_activate (RBPlugin *plugin,
 
 		rb_debug ("using tee-based visualization");
 
+		g_signal_connect_object (pi->player,
+					 "tee-inserted",
+					 G_CALLBACK (tee_visualizer_inserted),
+					 plugin, 0);
+		g_signal_connect_object (pi->player,
+					 "tee-pre-remove",
+					 G_CALLBACK (tee_visualizer_pre_remove),
+					 plugin, 0);
+
 		/* create the sink */
 		pi->video_sink = gst_element_factory_make ("gconfvideosink", "videosink");
 		gst_element_set_state (pi->video_sink, GST_STATE_READY);
