@@ -37,6 +37,7 @@
 
 static GObject *rb_auto_playlist_source_constructor (GType type, guint n_construct_properties,
 						      GObjectConstructParam *construct_properties);
+static void rb_auto_playlist_source_dispose (GObject *object);
 static void rb_auto_playlist_source_finalize (GObject *object);
 
 /* source methods */
@@ -113,6 +114,7 @@ rb_auto_playlist_source_class_init (RBAutoPlaylistSourceClass *klass)
 	RBPlaylistSourceClass *playlist_class = RB_PLAYLIST_SOURCE_CLASS (klass);
 
 	object_class->constructor = rb_auto_playlist_source_constructor;
+	object_class->dispose = rb_auto_playlist_source_dispose;
 	object_class->finalize = rb_auto_playlist_source_finalize;
 
 	source_class->impl_can_cut = (RBSourceFeatureFunc) rb_false_function;
@@ -138,9 +140,9 @@ rb_auto_playlist_source_init (RBAutoPlaylistSource *source)
 	if (playlist_pixbuf == NULL) {
 		gint size;
 
-		gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &size, NULL);
+		gtk_icon_size_lookup (RB_SOURCE_ICON_SIZE, &size, NULL);
 		playlist_pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
-							    GNOME_MEDIA_AUTO_PLAYLIST,
+							    RB_STOCK_AUTO_PLAYLIST,
 							    size,
 							    0, NULL);
 		if (playlist_pixbuf) {
@@ -159,18 +161,28 @@ rb_auto_playlist_source_init (RBAutoPlaylistSource *source)
 }
 
 static void
-rb_auto_playlist_source_finalize (GObject *object)
+rb_auto_playlist_source_dispose (GObject *object)
 {
 	RBAutoPlaylistSourcePrivate *priv = RB_AUTO_PLAYLIST_SOURCE_GET_PRIVATE (object);
 
 	if (priv->action_group != NULL) {
 		g_object_unref (priv->action_group);
+		priv->action_group = NULL;
 	}
 
 	if (priv->cached_all_query) {
 		g_object_unref (G_OBJECT (priv->cached_all_query));
+		priv->cached_all_query = NULL;
 	}
 
+	G_OBJECT_CLASS (rb_auto_playlist_source_parent_class)->dispose (object);
+}
+
+static void
+rb_auto_playlist_source_finalize (GObject *object)
+{
+	RBAutoPlaylistSourcePrivate *priv = RB_AUTO_PLAYLIST_SOURCE_GET_PRIVATE (object);
+
 	if (priv->query) {
 		rhythmdb_query_free (priv->query);
 	}
@@ -252,7 +264,7 @@ rb_auto_playlist_source_new (RBShell *shell, const char *name, gboolean local)
 					"shell", shell,
 					"is-local", local,
 					"entry-type", RHYTHMDB_ENTRY_TYPE_SONG,
-					"sourcelist-group", RB_SOURCELIST_GROUP_PERSISTANT,
+					"source-group", RB_SOURCE_GROUP_PLAYLISTS,
 					NULL));
 }
 
@@ -600,6 +612,7 @@ impl_save_contents_to_xml (RBPlaylistSource *psource,
 
 	rhythmdb_query_serialize (rb_playlist_source_get_db (psource), query, node);
 	rhythmdb_query_free (query);
+	g_value_array_free (limit_value);
 	g_free (sort_key);
 }
 
@@ -820,6 +833,11 @@ search_action_changed (GtkRadioAction  *action,
 	RBAutoPlaylistSource *source;
 
 	g_object_get (shell, "selected-source", &source, NULL);
+	if (source == NULL || !RB_IS_AUTO_PLAYLIST_SOURCE (source)) {
+		if (source != NULL)
+			g_object_unref (source);
+		return;
+	}
 
 	priv = RB_AUTO_PLAYLIST_SOURCE_GET_PRIVATE (source);
 
