@@ -42,7 +42,6 @@
 #include "rb-property-view.h"
 #include "rb-util.h"
 #include "rb-file-helpers.h"
-#include "totem-pl-parser.h"
 #include "rb-preferences.h"
 #include "rb-dialog.h"
 #include "rb-uri-dialog.h"
@@ -282,7 +281,7 @@ struct RBPodcastSourcePrivate
 
 static GtkActionEntry rb_podcast_source_actions [] =
 {
-	{ "MusicNewPodcast", GTK_STOCK_NEW, N_("_New Podcast Feed"), "<control>P",
+	{ "MusicNewPodcast", RB_STOCK_PODCAST_NEW, N_("_New Podcast Feed..."), "<control>P",
 	  N_("Subscribe to a new Podcast Feed"),
 	  G_CALLBACK (rb_podcast_source_cmd_new_podcast) },
 
@@ -380,7 +379,6 @@ static void
 rb_podcast_source_init (RBPodcastSource *source)
 {
 	GdkPixbuf *pixbuf;
-	GdkPixbuf *scaled = NULL;
 	gint       size;
 
 	source->priv = RB_PODCAST_SOURCE_GET_PRIVATE (source);
@@ -391,29 +389,16 @@ rb_podcast_source_init (RBPodcastSource *source)
 
 	gtk_container_add (GTK_CONTAINER (source), source->priv->vbox);
 
-	gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &size, NULL);
+	gtk_icon_size_lookup (RB_SOURCE_ICON_SIZE, &size, NULL);
 	pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
 					   RB_STOCK_PODCAST,
 					   size,
 					   0, NULL);
 
 	if (pixbuf != NULL) {
-		/* we only have this icon as a 48x48 bitmap, so it helps to
-		 * scale it to the right size.  otherwise the sourcelist gets
-		 * a bit uneven.
-		 */
-		gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &size, NULL);
-		scaled = gdk_pixbuf_scale_simple (pixbuf, size, size, GDK_INTERP_BILINEAR);
-
-		rb_source_set_pixbuf (RB_SOURCE (source), scaled);
-	}
-
-	if (pixbuf != NULL) {
+		rb_source_set_pixbuf (RB_SOURCE (source), pixbuf);
 		g_object_unref (pixbuf);
 	}
-	if (scaled != NULL) {
-		g_object_unref (scaled);
-	}
 }
 
 static void
@@ -601,7 +586,7 @@ rb_podcast_source_constructor (GType type,
 
 	rb_entry_view_append_column_custom (source->priv->posts, column,
 					    _("Date"), "Date",
-					    (GCompareDataFunc) rb_podcast_source_post_date_cell_sort_func, 0);
+					    (GCompareDataFunc) rb_podcast_source_post_date_cell_sort_func, 0, NULL);
 
 	rb_entry_view_append_column (source->priv->posts, RB_ENTRY_VIEW_COL_TITLE, TRUE);
 
@@ -622,7 +607,7 @@ rb_podcast_source_constructor (GType type,
 
 	rb_entry_view_append_column_custom (source->priv->posts, column,
 					    _("Feed"), "Feed",
-					    (GCompareDataFunc) rb_podcast_source_post_feed_cell_sort_func, 0);
+					    (GCompareDataFunc) rb_podcast_source_post_feed_cell_sort_func, 0, NULL);
 
 	rb_entry_view_append_column (source->priv->posts, RB_ENTRY_VIEW_COL_DURATION, FALSE);
 	rb_entry_view_append_column (source->priv->posts, RB_ENTRY_VIEW_COL_RATING, FALSE);
@@ -659,7 +644,7 @@ rb_podcast_source_constructor (GType type,
 
 	rb_entry_view_append_column_custom (source->priv->posts, column,
 					    _("Status"), "Status",
-					    (GCompareDataFunc) rb_podcast_source_post_status_cell_sort_func, 0);
+					    (GCompareDataFunc) rb_podcast_source_post_status_cell_sort_func, 0, NULL);
 
 	g_signal_connect_object (G_OBJECT (source->priv->posts),
 				 "sort-order-changed",
@@ -821,6 +806,7 @@ rb_podcast_source_new (RBShell *shell)
 					  "name", _("Podcasts"),
 					  "shell", shell,
 					  "entry-type", RHYTHMDB_ENTRY_TYPE_PODCAST_POST,
+					  "source-group", RB_SOURCE_GROUP_LIBRARY,
 					  NULL));
 
 	rb_shell_register_entry_type_for_source (shell, source,
@@ -1245,6 +1231,9 @@ impl_get_config_widget (RBSource *asource, RBShellPreferences *prefs)
 	source->priv->config_widget = glade_xml_get_widget (xml, "podcast_vbox");
 
 	btn_file = glade_xml_get_widget (xml, "location_chooser");
+	gtk_file_chooser_add_shortcut_folder (GTK_FILE_CHOOSER (btn_file),
+					      rb_music_dir (),
+					      NULL);
 	download_dir = rb_podcast_manager_get_podcast_dir (source->priv->podcast_mgr);
 	gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (btn_file),
 						 download_dir);
@@ -1608,7 +1597,7 @@ rb_podcast_source_feed_title_cell_data_func (GtkTreeViewColumn *column,
 		/* Subtract one for the All node */
 		nodes--;
 
-		fmt = ngettext ("All %d feed", "All %d feeds", nodes);
+		fmt = ngettext ("%d feed", "All %d feeds", nodes);
 
 		str = g_strdup_printf (fmt, nodes, number);
 	} else {
