@@ -153,9 +153,9 @@ rb_static_playlist_source_init (RBStaticPlaylistSource *source)
 {
 	if (playlist_pixbuf == NULL) {
 		gint size;
-		gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, &size, NULL);
+		gtk_icon_size_lookup (RB_SOURCE_ICON_SIZE, &size, NULL);
 		playlist_pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
-							    GNOME_MEDIA_PLAYLIST,
+							    RB_STOCK_PLAYLIST,
 							    size,
 							    0, NULL);
 		if (playlist_pixbuf) {
@@ -194,6 +194,12 @@ rb_static_playlist_source_dispose (GObject *object)
 
 	if (priv->filter_model != NULL) {
 		g_object_unref (priv->filter_model);
+		priv->filter_model = NULL;
+	}
+
+	if (priv->action_group != NULL) {
+		g_object_unref (priv->action_group);
+		priv->action_group = NULL;
 	}
 
 	G_OBJECT_CLASS (rb_static_playlist_source_parent_class)->dispose (object);
@@ -208,10 +214,6 @@ rb_static_playlist_source_finalize (GObject *object)
 
 	g_free (priv->search_text);
 
-	if (priv->action_group != NULL) {
-		g_object_unref (priv->action_group);
-	}
-
 	G_OBJECT_CLASS (rb_static_playlist_source_parent_class)->finalize (object);
 }
 
@@ -294,7 +296,7 @@ rb_static_playlist_source_new (RBShell *shell, const char *name, gboolean local,
 					"shell", shell,
 					"is-local", local,
 					"entry-type", entry_type,
-					"sourcelist-group", RB_SOURCELIST_GROUP_PERSISTANT,
+					"source-group", RB_SOURCE_GROUP_PLAYLISTS,
 					NULL));
 }
 
@@ -824,6 +826,12 @@ search_action_changed (GtkRadioAction  *action,
 	gboolean active;
 
 	g_object_get (shell, "selected-source", &source, NULL);
+	if (source == NULL || !RB_IS_STATIC_PLAYLIST_SOURCE (source)) {
+		if (source != NULL)
+			g_object_unref (source);
+		return;
+	}
+
 	priv = RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE (source);
 
 	active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (current));
